@@ -1,4 +1,4 @@
-from options import *
+from .options import *
 
 from mongoadmin.sites import site
 
