@@ -3,9 +3,10 @@ import collections
 from django import forms, template
 from django.forms.formsets import all_valid
 from django.forms.models import modelformset_factory
-from django.contrib.contenttypes.models import ContentType
+from mongoadmin.contenttypes.models import ContentType
 from django.contrib.admin import widgets, helpers
 from django.contrib.admin.util import unquote, flatten_fieldsets, model_format_dict
+from django.contrib.admin.widgets import ManyToManyRawIdWidget
 from django.contrib import messages
 from django.views.decorators.csrf import csrf_protect
 from django.core.exceptions import PermissionDenied, ValidationError
@@ -39,14 +40,14 @@ from mongoengine.fields import (DateTimeField, URLField, IntField, ListField, Em
                                 ReferenceField, StringField, FileField, ImageField)
 
 from mongodbforms.documentoptions import DocumentMetaWrapper
-from mongoadmin import mongohelpers
-from mongoadmin.util import RelationWrapper
-
 from mongodbforms.documents import (documentform_factory, DocumentForm, EmbeddedDocumentForm,
                                   inlineformset_factory, BaseInlineDocumentFormSet, 
-                                  embeddedformset_factory, EmbeddedDocumentFormSet)
-from mongodbforms import MongoDefaultFormFieldGenerator, save_instance
-from mongodbforms.util import with_metaclass
+                                  embeddedformset_factory, EmbeddedDocumentFormSet, save_instance)
+from mongodbforms.util import with_metaclass, load_field_generator
+
+from mongoadmin import mongohelpers
+from mongoadmin.util import RelationWrapper, is_django_user_model
+from mongoadmin.widgets import ReferenceRawIdWidget, MultiReferenceRawIdWidget
 
 HORIZONTAL, VERTICAL = 1, 2
 # returns the <ul> class for a given radio_admin field
@@ -72,6 +73,9 @@ FORMFIELD_FOR_DBFIELD_DEFAULTS = {
 
 csrf_protect_m = method_decorator(csrf_protect)
 
+_fieldgenerator = load_field_generator()
+_fieldgenerator = _fieldgenerator()
+
 def formfield(field, form_class=None, **kwargs):
     """
     Returns a django.forms.Field instance for this database Field.
@@ -84,9 +88,6 @@ def formfield(field, form_class=None, **kwargs):
         else:
             defaults['initial'] = field.default
             
-    if hasattr(field, 'max_length') and field.choices is None:
-        defaults['max_length'] = field.max_length
-            
     if field.choices is not None:
         # Many of the subclass-specific formfield arguments (min_value,
         # max_value) don't apply for choice fields, so be sure to only pass
@@ -101,8 +102,7 @@ def formfield(field, form_class=None, **kwargs):
     
     if form_class is not None:
         return form_class(**defaults)
-    else:
-        return MongoDefaultFormFieldGenerator().generate(field, **defaults)
+    return _fieldgenerator.generate(field, **defaults)
 
     
 class BaseDocumentAdmin(with_metaclass(forms.MediaDefiningClass, object)):
@@ -133,7 +133,8 @@ class BaseDocumentAdmin(with_metaclass(forms.MediaDefiningClass, object)):
         # passed to formfield_for_dbfield override the defaults.
         for klass in db_field.__class__.mro():
             if klass in self.formfield_overrides:
-                kwargs = dict(self.formfield_overrides[klass], **kwargs)
+                kwargs.update(self.formfield_overrides[klass])
+                break
         return formfield(db_field, **kwargs)        
 
     def formfield_for_dbfield(self, db_field, **kwargs):
@@ -168,7 +169,10 @@ class BaseDocumentAdmin(with_metaclass(forms.MediaDefiningClass, object)):
                             form_field.widget, RelationWrapper(db_field.document_type), self.admin_site,
                             can_add_related=can_add_related)
                 return form_field
-
+            elif db_field.name in self.raw_id_fields:
+                kwargs['widget'] = ReferenceRawIdWidget(db_field.rel, self.admin_site)
+                return self._get_formfield(db_field, **kwargs)
+                
         if isinstance(db_field, StringField):
             if db_field.max_length is None:
                 kwargs = dict({'widget': widgets.AdminTextareaWidget}, **kwargs)
@@ -205,7 +209,7 @@ class BaseDocumentAdmin(with_metaclass(forms.MediaDefiningClass, object)):
         db = kwargs.get('using')
 
         if db_field.name in self.raw_id_fields:
-            kwargs['widget'] = widgets.ManyToManyRawIdWidget(db_field.rel, using=db)
+            kwargs['widget'] = MultiReferenceRawIdWidget(db_field.field.rel, self.admin_site)
             kwargs['help_text'] = ''
         elif db_field.name in (list(self.filter_vertical) + list(self.filter_horizontal)):
             kwargs['widget'] = widgets.FilteredSelectMultiple(pretty_name(db_field.name), (db_field.name in self.filter_vertical))
@@ -354,7 +358,7 @@ class DocumentAdmin(BaseDocumentAdmin):
     @property
     def content_type(self):
         name = app_label, model = self.opts.app_label, self.opts.module_name
-        name = '%s %s'%name
+        name = '%s %s' % name
         ret = ContentType.objects.get_or_create(name=name, app_label=app_label, model=model)
         return ret[0]
 
@@ -584,6 +588,9 @@ class DocumentAdmin(BaseDocumentAdmin):
 
         The default implementation creates an admin LogEntry object.
         """
+        if not is_django_user_model(request.user):
+            return
+            
         from django.contrib.admin.models import LogEntry, ADDITION
         LogEntry.objects.log_action(
             user_id         = request.user.pk,
@@ -599,6 +606,9 @@ class DocumentAdmin(BaseDocumentAdmin):
 
         The default implementation creates an admin LogEntry object.
         """
+        if not is_django_user_model(request.user):
+            return
+        
         from django.contrib.admin.models import LogEntry, CHANGE
         LogEntry.objects.log_action(
             user_id         = request.user.pk,
@@ -616,6 +626,9 @@ class DocumentAdmin(BaseDocumentAdmin):
 
         The default implementation creates an admin LogEntry object.
         """
+        if not is_django_user_model(request.user):
+            return
+        
         from django.contrib.admin.models import LogEntry, DELETION
         LogEntry.objects.log_action(
             user_id         = request.user.id,
@@ -812,7 +825,7 @@ class DocumentAdmin(BaseDocumentAdmin):
             'ordered_objects': ordered_objects,
             'form_url': mark_safe(form_url),
             'opts': opts,
-            #'content_type_id': ContentType.objects.get_for_model(self.model).id,
+            'content_type_id': str(self.content_type.id),
             'save_as': self.save_as,
             'save_on_top': self.save_on_top,
             'root_path': self.admin_site.root_path,
@@ -825,7 +838,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         return render_to_response(form_template or [
             "admin/%s/%s/change_form.html" % (app_label, opts.object_name.lower()),
             "admin/%s/change_form.html" % app_label,
-            "admin/change_form.html"
+            "admin/mongo_change_form.html"
         ], context, context_instance=context_instance)
 
     def response_add(self, request, obj, post_url_continue='../%s/'):
@@ -970,7 +983,6 @@ class DocumentAdmin(BaseDocumentAdmin):
         
 
     @csrf_protect_m
-    @transaction.commit_on_success
     def add_view(self, request, form_url='', extra_context=None):
         "The 'add' admin view for this model."
         model = self.document
@@ -1096,7 +1108,6 @@ class DocumentAdmin(BaseDocumentAdmin):
         return self.render_change_form(request, context, form_url=form_url, add=True)
 
     @csrf_protect_m
-    @transaction.commit_on_success
     def change_view(self, request, object_id, extra_context=None):
         "The 'change' admin view for this model."
         model = self.document
