@@ -2,7 +2,6 @@ from django import http, template
 from django.contrib.admin import ModelAdmin
 from django.contrib.admin.forms import AdminAuthenticationForm
 from django.contrib.auth import REDIRECT_FIELD_NAME
-from django.contrib.contenttypes import views as contenttype_views
 from django.views.decorators.csrf import csrf_protect
 from django.db.models.base import ModelBase
 from django.core.exceptions import ImproperlyConfigured
@@ -18,7 +17,11 @@ from django.contrib.admin.sites import NotRegistered, AlreadyRegistered
 
 from mongoengine.base import TopLevelDocumentMetaclass
 
+from mongodbforms import init_document_options
+
 from mongoadmin import actions, DocumentAdmin
+from mongoadmin.util import is_django_user_model
+from mongoadmin.contenttypes import views as contenttype_views
 
 LOGIN_FORM_KEY = 'this_is_the_login_form'
 
@@ -65,6 +68,7 @@ class AdminSite(object):
         """
         if isinstance(model_or_iterable, ModelBase) and not admin_class:
             admin_class = ModelAdmin
+            
         if isinstance(model_or_iterable, TopLevelDocumentMetaclass) and not admin_class:
             admin_class = DocumentAdmin
 
@@ -79,6 +83,9 @@ class AdminSite(object):
             model_or_iterable = [model_or_iterable]
 
         for model in model_or_iterable:
+            if isinstance(model, TopLevelDocumentMetaclass):
+                init_document_options(model)
+            
             if hasattr(model._meta, 'abstract') and model._meta.abstract:
                 raise ImproperlyConfigured('The model %s is abstract, so it '
                       'cannot be registered with admin.' % model.__name__)
@@ -86,6 +93,11 @@ class AdminSite(object):
             if model in self._registry:
                 raise AlreadyRegistered('The model %s is already registered' % model.__name__)
 
+            # Ignore the registration if the model has been
+            # swapped out.
+            if model._meta.swapped:
+                continue
+
             # If we got **options then dynamically construct a subclass of
             # admin_class with those **options.
             if options:
@@ -234,8 +246,9 @@ class AdminSite(object):
             url(r'^jsi18n/$',
                 wrap(self.i18n_javascript, cacheable=True),
                 name='jsi18n'),
-            url(r'^r/(?P<content_type_id>\d+)/(?P<object_id>.+)/$',
-                wrap(contenttype_views.shortcut)),
+            url(r'^r/(?P<content_type_id>.+)/(?P<object_id>.+)/$',
+                wrap(contenttype_views.shortcut),
+                name='view_on_site'),
             url(r'^(?P<app_label>\w+)/$',
                 wrap(self.app_index),
                 name='app_list')
@@ -402,7 +415,12 @@ class AdminSite(object):
         }
         context.update(extra_context or {})
         context_instance = template.RequestContext(request, current_app=self.name)
-        return render_to_response(self.index_template or 'admin/index.html', context,
+
+        if is_django_user_model(user):
+            default_template = 'admin/index.html'
+        else:
+            default_template = 'admin/mongo_user_index.html'
+        return render_to_response(self.index_template or default_template, context,
             context_instance=context_instance
         )
 
